<?php
// $Id: nd.admin.inc,v 1.1.2.17 2009/08/11 19:42:08 swentel Exp $

/**
 * @file
 * Administrative functions for node displays module.
 */

/**
 * Field overview form, loads the original field overview form of CCK.
 */
function nd_field_overview_form($type) {
  module_load_include('inc', 'content', 'includes/content.admin');
  return drupal_get_form('content_field_overview_form', $type);
}

/**
 * Menu callback; presents a listing of fields display settings for a content type.
 *
 * Form includes form widgets to select which fields appear for teaser, full node
 * etc and how the field labels should be rendered.
 *
 * This function is based on content_display_overview_form from
 * content.admin.inc, but altered to have weights and regions.
 */
function nd_content_display_overview_form(&$form_state, $type_name, $context = 'nd_node') {
  module_load_include('inc', 'content', 'includes/content.admin');

  content_inactive_message($type_name);

  // Gather type information.
  $type = content_types($type_name);
  $field_types = _content_field_types();
  $fields = $type['fields'];

  $groups = $group_options = array();
  if (module_exists('fieldgroup')) {
    $groups = fieldgroup_groups($type['type']);
    $group_options = _fieldgroup_groups_label($type['type']);
  }

  $form = array(
    '#tree' => TRUE,
    '#type_name' => $type['type'],
    '#has_body' => $type['has_body'],
    '#fields' => array_keys($fields),
    '#groups' => array_keys($groups),
    '#context' => $context,
  );

  // Exclude build mode.
  $preprocess = variable_get('nd_preprocess_exclude', array());
  $excluded = (isset($preprocess[$type_name][$context]) && $preprocess[$type_name][$context] == TRUE) ? TRUE : FALSE;
  $form['exclude_build_mode'] = array(
    '#type' => 'checkbox',
    '#title' => t('Exclude this build mode from rendering.'),
    '#description' => t('The node object will still have this build mode property, so you can make exceptions in your template file.'),
    '#default_value' => $excluded,
  );

  // Fields
  // @TODO Fix the default values ... Appearantly some old code is blocking the default values
  // from being inserted correctly. node =/= full, teaser is fine, sticky has no information as well
  // Ask Stalski for this problem if you can't figure out.
  // Another problem too is that cck uses the exclude value too, we need to fix that somehow
  // since we ditched that now with the disabled region.
  foreach ($fields as $name => $field) {
    $field_type = $field_types[$field['type']];
    $defaults = $field['display_settings'];
    $weight = isset($field['display_settings'][$context]['weight']) ? $field['display_settings'][$context]['weight'] : 0;

    $form[$name] = array(
      'human_name' => array('#value' => check_plain($field['widget']['label'])),
      'weight' => array('#type' => 'value', '#default_value' => $weight),
      'parent' => array('#type' => 'value', '#value' => ''),
    );

    $form[$name]['weight'] = array('#type' => 'value', '#default_value' => $weight);
    $form[$name]['nd_weight'] = array(
      '#type' => 'weight',
      '#default_value' => $weight,
      '#delta' => 20,
    );

    $form[$name][$context] = _nd_display_field_default();

    if (isset($defaults[$context]['label']['format'])) {
      $form[$name][$context]['label']['format']['#default_value'] = $defaults[$context]['label']['format'];
    }

    // Formatters specific to field types.
    $options = nd_field_default_formatters();
    foreach ($field_type['formatters'] as $formatter_name => $formatter_info) {
      $options[$formatter_name] = $formatter_info['label'];
    }
    $form[$name][$context]['format']['#options'] = $options;
    if (isset($defaults[$context]['format'])) {
      $form[$name][$context]['format']['#default_value'] = $defaults[$context]['format'];
    }
    if (isset($defaults[$context]['region'])) {
      $form[$name][$context]['region']['#default_value'] =  $defaults[$context]['region'];
      $form[$name][$context]['region']['#attributes']['class'] = 'field-region-select field-region-'. $defaults[$context]['region'];
    }

  }

  // Groups.
  $label_options = array(
    'above' => t('Above'),
    'hidden' => t('<Hidden>'),
  );
  $options = array(
    'no_style' => t('no styling'),
    'simple' => t('simple'),
    'fieldset' => t('fieldset'),
    'fieldset_collapsible' => t('fieldset - collapsible'),
    'fieldset_collapsed' => t('fieldset - collapsed'),
    'hidden' => t('<Hidden>'),
  );
  foreach ($groups as $name => $group) {
    $defaults = $group['settings']['display'];
    $weight = $group['weight'];

    $form[$name] = array(
      'human_name' => array('#value' => check_plain($group['label'])),
      'weight' => array('#type' => 'value', '#value' => $weight),
    );
    if ($context == 'basic') {
      $form[$name]['label'] = array(
        '#type' => 'select',
        '#options' => $label_options,
        '#default_value' => isset($defaults['label']) ? $defaults['label'] : 'above',
      );
    }
    $form[$name][$context]['format'] = array(
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => isset($defaults[$context]['format']) ? $defaults[$context]['format'] : 'fieldset',
    );
    foreach ($group['fields'] as $field_name => $field) {
      $form[$field_name]['parent']['#value'] = $name;
    }
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  return $form;
}

/**
 * Submit handler for the display overview form.
 */
function nd_content_display_overview_form_submit($form, &$form_state) {

  $context = $form['#context'];

  // Exclude build mode.
  $exclude_build_mode  = $form_state['values']['exclude_build_mode'];
  $preprocess = variable_get('nd_preprocess_exclude', array());
  $preprocess[$form['#type_name']][$context] = $exclude_build_mode;
  variable_set('nd_preprocess_exclude', $preprocess);

  // Fields.
  module_load_include('inc', 'content', 'includes/content.crud');
  $form_values = $form_state['values'];
  foreach ($form_values as $key => $values) {
    // Groups are handled in fieldgroup_display_overview_form_submit().
    if (in_array($key, $form['#fields'])) {
      $field = content_fields($key, $form['#type_name']);
      // We have some numeric keys here, so we can't use array_merge.
      $field['display_settings'] = $values + $field['display_settings'];
      $field['display_settings'][$context]['exclude'] = ($field['display_settings'][$context]['region'] == 'disabled') ? 1 : 0;
      $field['display_settings'][$context]['weight'] = $form_values[$key]['nd_weight'];
      content_field_instance_update($field);
    }
  }

  drupal_set_message(t('Your settings have been saved.'));
}

/**
 * Add extra fields.
 */
function nd_add_extra_fields(&$form) {

  $context = $form['#context'];
  $type = $form['#type_name'];

  // Add has_body property to the form.
  $node_type = node_get_types('type', $type);
  $has_body = $node_type->has_body;

  // our submit first.
  $submit = $form['#submit'][0];

  // Get extra fields.
  $fields = array();
  $fields = module_invoke_all('nd_extra_display_fields', $type, $has_body);
  if (!empty($fields)) {
    foreach ($fields as $field => $value) {
      $form['#fields'][] = $field;
      $form[$field] = $form[$form['#fields'][0]];
      $form[$field]['human_name']['#value'] = $value['title'];
      $form[$field]['weight']['#default_value'] = variable_get('display_'. $field .'_'. $context .'_weight_'. $type, -19);
      $form[$field]['nd_weight']['#default_value'] = variable_get('display_'. $field .'_'. $context .'_weight_'. $type, -19);
      // Parent is empty currently, this will change in the future normally.
      $form[$field]['parent']['#value'] = '';
      $form[$field][$context]['label']['format']['#access'] = isset($value['label']) ? TRUE : FALSE;
      $form[$field][$context]['label']['format']['#default_value'] = 'hidden';
      $form[$field][$context]['format']['#options'] = isset($value['formatters']) ? $value['formatters'] : array('none');
      $form[$field][$context]['format']['#default_value'] = isset($field['formatters_default']) ? variable_get('display_'. $field .'_'. $context .'_format_'. $type, $field['formatters_default']) : '';
      $form[$field][$context]['format']['#access'] = isset($value['formatters']) ? TRUE : FALSE;
      $form[$field][$context]['region']['#default_value'] = variable_get('display_'. $field .'_'. $context .'_region_'. $type, 'middle');
    }
  }

  $form['#submit'][0] = 'nd_add_extra_fields_submit';
  $form['#submit'][] = $submit;

}

/**
 * Save the extra cck display fields per context.
 */
function nd_add_extra_fields_submit($form, &$form_state) {

  $type = $form['#type_name'];
  $context = $form['#context'];

  $fields = module_invoke_all('nd_extra_display_fields', $form['#node_type'], $form['#has_body']);
  if (!empty($fields)) {
    foreach ($fields as $field => $value) {
      variable_set('display_'. $field .'_'. $context .'_weight_'. $type, $form_state['values'][$field]['nd_weight']);
      variable_set('display_'. $field .'_'. $context .'_format_'. $type, $form_state['values'][$field][$context]['format']);
      variable_set('display_'. $field .'_'. $context .'_region_'. $type, $form_state['values'][$field][$context]['region']);
      // Don't let cck submit flip on it.
      unset($form_state['values'][$field]);
    }
  }
}

/**
 * Settings page.
 */
function nd_settings($form_state) {
  $form = array();

  $form['nd_overrule_search'] = array(
    '#type' => 'checkbox',
    '#title' => t('Override default search callback to use Node displays.'),
    '#description' => t('This only applies if the core search module is enabled.'),
    '#default_value' => variable_get('nd_overrule_search', FALSE),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );

  return $form;
}

/**
 * Settings page submit.
 */
function nd_settings_submit($form_id, &$form_state) {
  drupal_set_message(t('The configuration options have been saved.'));
  variable_set('nd_overrule_search', $form_state['values']['nd_overrule_search']);
  menu_rebuild();
}

/**
 * Custom displays.
 */
function nd_custom_displays() {
  $output = '';
  $action = arg(4);
  $display = arg(5);
  $custom_displays = variable_get('nd_custom_displays', array());

  // Delete form.
  if (in_array($display, array_keys($custom_displays)) && $action == 'delete') {
    $output .= drupal_get_form('nd_custom_display_delete_form', $display);
  }
  // Edit form.
  elseif (in_array($display, array_keys($custom_displays)) && $action == 'edit') {
    $output .= drupal_get_form('nd_custom_display_edit_form', $display);
  }
  // Overview form.
  else {
    $output .= '<p>'. t('Manage your custom displays.') .'</p>';

    // Custom content types.
    if (empty($custom_displays)) {
      $output .= '<p>'. t('You have not defined any custom displays.') .'</p>';
    }
    else {
      $header = array(
        t('Name'), t('Operations'),
      );

      $rows = array();
      foreach ($custom_displays as $key => $name) {
        $row = array();
        $row[] = check_plain($name);
        $row[] = l(t('Edit'), 'admin/content/types/displays/edit/'. $key) .' - '. l(t('Delete'), 'admin/content/types/displays/delete/'. $key);
        $rows[] = $row;
      }

      $output .= theme('table', $header, $rows);
    }

    // New content type form.
    $output .= drupal_get_form('nd_custom_display_form');
  }

  return $output;
}

/**
 * Custom display form.
 */
function nd_custom_display_form($form_state) {
  $form = array();

  $form['identity'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add new display'),
  );
  $form['identity']['key'] = array(
    '#type' => 'textfield',
    '#title' => 'Key name',
    '#description' => t('The machine-readable name of this display.'),
    '#required' => TRUE,
  );
  $form['identity']['name'] = array(
    '#type' => 'textfield',
    '#title' => 'Display name',
    '#description' => t('The name of this display which will be used for every content type on the display fields page.'),
    '#required' => TRUE,
  );
  $form['identity']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save custom display'),
  );

  return $form;
}

/**
 * Validate new custom display submission.
 */
function nd_custom_display_form_validate($form_id, &$form_state) {
  $existing = array_keys(nd_get_build_modes());

  if (in_array($form_state['values']['key'], $existing)) {
    form_set_error('key', t('This display mode already exists.'));
  }

  if (!preg_match('!^[a-z]+$!', $form_state['values']['key'])) {
    form_set_error('key', t('The machine-readable name must contain only lowercase letters.'));
  }

}

/**
 * Create new custom display.
 */
function nd_custom_display_form_submit($form_id, &$form_state) {
  $custom_displays = variable_get('nd_custom_displays', array());
  $custom_displays[$form_state['values']['key']] = $form_state['values']['name'];
  variable_set('nd_custom_displays', $custom_displays);
  variable_set('nd_all_displays', array());
  menu_rebuild();
}

/**
 * Custom display edit form.
 */
function nd_custom_display_edit_form($form_state, $display) {
  $form = array();

  $custom_displays = variable_get('nd_custom_displays', array());
  $title = $custom_displays[$display];

  $form['identity'] = array(
    '#type' => 'fieldset',
    '#title' => t('Edit display'),
  );
  $form['identity']['key'] = array(
    '#type' => 'textfield',
    '#title' => 'Key name',
    '#description' => t('The machine-readable name of this display. Note, you can not edit this key.'),
    '#required' => TRUE,
    '#disabled' => TRUE,
    '#value' => $display,
  );
  $form['identity']['name'] = array(
    '#type' => 'textfield',
    '#title' => 'Display name',
    '#default_value' => $title,
    '#description' => t('The name of this display which will be used for every content type on the display fields page.'),
    '#required' => TRUE,
  );
  $form['identity']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update custom display'),
  );

  return $form;
}

/**
 * Update custom display.
 */
function nd_custom_display_edit_form_submit($form_id, &$form_state) {
  $custom_displays = variable_get('nd_custom_displays', array());
  $custom_displays[$form_state['values']['key']] = $form_state['values']['name'];
  variable_set('nd_custom_displays', $custom_displays);
  variable_set('nd_all_displays', array());
  menu_rebuild();
  drupal_set_message(t('Custom display %display has been updated.', array('%display' => $custom_displays[$form_state['values']['key']])));
  $form_state['redirect'] = 'admin/content/types/displays';
}

/**
 * Custom display delete form.
 */
function nd_custom_display_delete_form($form_state, $display) {
  $form = array();
  $form['question'] = array(
    '#type' => 'markup',
    '#value' => '<p>'. t('Are you sure you want to delete the display type %display ?', array('%display' => $display)) .'</p>',
  );
  $form['display'] = array(
    '#type' => 'value',
    '#value' => $display,
  );
  $form['button']['submit'] = array(
    '#prefix' => '<div>',
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  $form['buttons']['cancel'] = array(
    '#suffix' => '</div>',
    '#type' => 'markup',
    '#value' => l('cancel', 'admin/content/types/displays'),
  );
  return $form;
}

/**
 * Delete callback: delete display.
 */
function nd_custom_display_delete_form_submit($form_id, &$form_state) {
  $custom_displays = variable_get('nd_custom_displays', array());
  unset($custom_displays[$form_state['values']['display']]);
  variable_set('nd_custom_displays', $custom_displays);
  variable_set('nd_all_displays', array());
  menu_rebuild();
  drupal_set_message(t('Custom display %display has been removed.', array('%display' => $form_state['values']['display'])));
  $form_state['redirect'] = 'admin/content/types/displays';
}
